#
# Slide files are simply tar files with slides numbered 1,2,3...
#
# variables:
# ----------
# sld(page,current) - current slide (integer)
# sld(page,max) - last slide in the tarball
# sld(page,update) - flags if the slide need to be flushed
#   *** procs that turn on update flag: objectStartMove, objectOptions,
#	cutObject, pasteObject, frontObject, backObject, groupObjects, 
#	ungroupObjects, insertLine, insertTextMark, insertImage.
#	(be aware of this when introducing another editing procedure)
# The directory $sld(dir)/tmp is used for temporary storage
# of slide pages.
#

#
# fileNew
#
proc fileNew {} {
	global sld sldi cv
	.c.c delete slide
	set sld(page,update) 0
	set sld(page,max) 1
	set sld(page,current) 1
	set sld(update) 0
	cleanSlideDir
	statusSlide
}

#
# fileSave -- try to save the file, guessing its name or asking the user
#
proc fileSave {} {
	global sld sldi cv
	updateSlide $sld(page,current)
# if no filename found, try "noname"
	if {![info exists sld(currfile)]} {
		set sld(currfile) noname
	} 
# ask the user for its name
	regexp "(.*/)(.*)" $sld(currfile) fullname path fname
	set fn [tk_getSaveFile -initialdir $path -initialfile $fname]
	set sld(currfile) $fn
# add all slide files to the tarball
	set olddir [exec pwd]
	cd $sld(dir)/tmp
	eval exec tar cvzf $fn [glob *]
	cd $olddir
# updates file list menu
	fileListUpdate $fn
}

#
# fileMerge -- Merge loaded file with the present drawing, so it becomes 
# easier to make several similar slides :)
#
proc fileMerge {} {
	global sld sldi cv
#	source [tk_getOpenFile]	
#	.c.c config -bg $sld(background)
}

#
# fileLoad -- clean slide, get a filename and "source" it.
# Then update the list of files recently loaded. 
#
proc fileLoad { { fn "" } } {
	global sld sldi cv
# if no filename found, try "noname"
	if {![info exists sld(currfile)]} {
		set sld(currfile) noname
	} 
	if {[string length $fn] == 0} { set fn [tk_getOpenFile -initialfile $sld(currfile)] }
### if still no name, user has canceled the loading
	if {[string length $fn] == 0} { return }
	fileNew
### expand tarball to tmp directory
	set olddir [exec pwd]
	cd $sld(dir)/tmp
	eval exec tar xvzf $fn
	cd $olddir
	set sld(currfile) $fn
	fileListUpdate $fn
### update sld(page,max)
	set sld(page,max) [llength [glob $sld(dir)/tmp/*]]
	set sld(page,current) 1
	showSlide
}

#
# show one slide or all slides if in multislide view
#
proc showSlide { } {
	global sld cv
	if {$sld(multi)} {
		set n 1
		for {set i 0} {$i < 3} {incr i} {
			for {set j 0} {$j < 2} {incr j} {
				if {$n > $sld(page,max)} {
					break
				}
				set cv .c.x${i}y$j
				showOneSlide $n
				incr n
				update idletasks
			}
		}
	} else {
		set cv .c.c
		showOneSlide $sld(page,current)
	}
}

proc cleanSlideDir { } {
	global sld sldi cv
	catch {
		foreach f [glob $sld(dir)/tmp/*] {
			exec rm $f
		}
	}
}

proc updateSlide { n } {
	global sld sldi cv
	if {$sld(page,update)} {
		set sld(update) 1
		set fd [open $sld(dir)/tmp/$n w]
# get each object description and write to the slide file
		foreach i [.c.c find withtag slide] {
			puts $fd [describeObj $i]
		}
# save also all variables in sld(svarlist), that describes some properties
# of this particular slide
		foreach i $sld(svarlist) {
			puts $fd "set sld($i) \"$sld($i)\""
		}
		close $fd
		set sld(page,update) 0
	}
# erase all selected objects (for grouping)
	$cv delete withtag flaggrp
	set sld(group) {}
}

proc showOneSlide { n } {
	global sld sldi sldimg sldfnt cv
	$cv delete slide
	catch {unset sldi}
	catch {unset sldimg}
	catch {unset sldfnt}
	if {[file exists $sld(dir)/tmp/$n]} {
		source $sld(dir)/tmp/$n
		$cv config -bg $sld(background)
		if {$sld(multi)} {
			$cv scale slide 30 30 $sld(scale) $sld(scale)
			foreach i [$cv find withtag slide] {
				switch 	[$cv type $i] {
				"text" {
					set fnt [$cv itemcget $i -font]
					set fsiz [expr round([lindex $fnt 1] * $sld(scale))]
					set fnt [lreplace $fnt 1 1 $fsiz] 
					$cv itemconfigure $i -font $fnt
				}
				"image" {
					set img [$cv itemcget $i -image]
					image create photo z$img
					if {$sld(scale) < 1.0} {
						z$img copy $img -subsample [expr round(1/$sld(scale))]
					} else {
						set zscale [expr round($sld(scale))]
						z$img copy $img -zoom $zscale $zscale						
					}
					$cv itemconfigure $i -image z$img
				}	
				}		
			}
		} else {
			$cv scale slide 0 0 $sld(scale) $sld(scale)
			foreach i [$cv find withtag slide] {
				switch 	[$cv type $i] {
				"text" {
					if [array exists sldfnt($i)] {
						set fnt $sldfnt($i)
					} else {
						set fnt [$cv itemcget $i -font]
					}
					set fsiz [expr round([lindex $fnt 1] * $sld(scale))]
					set showfnt [lreplace $fnt 1 1 $fsiz] 
					$cv itemconfigure $i -font $showfnt
					set sldfnt($i) $fnt
				}
				"image" {
					if [info exists sldimg($i)] {
						set img $sldimg($i)
					} else {
						set img [$cv itemcget $i -image]
					}
					catch { image delete z$img }
					image create photo z$img
					if {$sld(scale) < 1.0} {
						z$img copy $img -subsample [expr round(1/$sld(scale))]
					} else {
						set zscale [expr round($sld(scale))]
						z$img copy $img -zoom $zscale $zscale						
					}
					$cv itemconfigure $i -image z$img
					set sldimg($i) $img
				}	
				}		
			}
			$cv config -height $sld(height) -width $sld(width) \
				-scrollregion [list -1c -1c \
					[expr round($sld(dwidth)*$sld(scale))] \
					[expr round($sld(dheight)*$sld(scale))]]
		}
	}
	statusSlide
}

#
# nextSlide, prevSlide -- navigation events for control with the mouse
#
proc gotoSlide { n } {
	global sld sldi cv
	if {$n < $sld(page,max)} {
		updateSlide $sld(page,current)
		set sld(page,current) $n
		showOneSlide $sld(page,current)
	}
}

proc nextSlide { } {
	global sld sldi cv
	updateSlide $sld(page,current)
	incr sld(page,current)	
	if {$sld(page,current) > $sld(page,max)} {
		incr sld(page,max)
	}
	showOneSlide $sld(page,current)
}

proc prevSlide { } {
	global sld sldi cv
	if {$sld(page,current) == 1} { return }
	updateSlide $sld(page,current)
	incr sld(page,current) -1
	showOneSlide $sld(page,current)
}

proc statusSlide { } {
	global sld sldi cv
	set s [format "slide: $sld(page,current)/$sld(page,max)"]
	.cnt config -text $s
}
